@@ -30,6 +30,7 @@
 #include <tests/testfile.h>
 #include <language/duchain/declaration.h>
 #include <language/duchain/problem.h>
+#include <language/duchain/types/integraltype.h>
 
 QTEST_MAIN(Php::TestDUChainMultipleFiles)
 
@@ -79,10 +80,9 @@ void TestDUChainMultipleFiles::testImportsBaseClassNotYetParsed()
     f2.parse(features);
 
     TestFile f1(QStringLiteral("<? class A {}"), QStringLiteral("php"), project);
-    f1.parse(features, 100); //low priority, to make sure f2 is parsed first
+    f1.parseAndWait(features, 100, 2000); //low priority, to make sure f2 is parsed first
 
-    QVERIFY(f1.waitForParsed());
-    QTest::qWait(100);
+    QVERIFY(f1.isReady());
 
     DUChainWriteLocker lock(DUChain::lock());
     QVERIFY(f2.topContext()->imports(f1.topContext(), CursorInRevision(0, 0)));
@@ -117,14 +117,12 @@ void TestDUChainMultipleFiles::testImportsGlobalFunctionNotYetParsed()
     f2.parse(features);
 
     TestFile f1(QStringLiteral("<? function foo2() {}"), QStringLiteral("php"), project);
-    f1.parse(features, 100); //low priority, to make sure f2 is parsed first
+    f1.parseAndWait(features, 100, 2000); //low priority, to make sure f2 is parsed first
 
-    QVERIFY(f2.waitForParsed());
-    QTest::qWait(100);
+    QVERIFY(f1.isReady());
 
     DUChainWriteLocker lock(DUChain::lock());
     QVERIFY(f2.topContext()->imports(f1.topContext(), CursorInRevision(0, 0)));
-
 }
 
 void TestDUChainMultipleFiles::testNonExistingGlobalFunction()
@@ -155,10 +153,9 @@ void TestDUChainMultipleFiles::testImportsStaticFunctionNotYetParsed()
     f2.parse(features);
 
     TestFile f1(QStringLiteral("<? class C { public static function foo() {} }"), QStringLiteral("php"), project);
-    f1.parse(features, 100); //low priority, to make sure f2 is parsed first
+    f1.parseAndWait(features, 100, 2000); //low priority, to make sure f2 is parsed first
 
-    QVERIFY(f2.waitForParsed());
-    QTest::qWait(100);
+    QVERIFY(f1.isReady());
 
     DUChainWriteLocker lock(DUChain::lock());
     QVERIFY(f2.topContext()->imports(f1.topContext(), CursorInRevision(0, 0)));
@@ -294,3 +291,30 @@ void TestDUChainMultipleFiles::testTodoExtractorReparse()
         QCOMPARE(top->problems().at(0)->range(), RangeInRevision(2, 3, 2, 7));
     }
 }
+
+void TestDUChainMultipleFiles::testIteratorForeachReparse() {
+    TestFile file(QStringLiteral("<?php\n/*\n\n/*\n*/\nforeach (new A() as $a) {}\nclass A implements Iterator {\npublic function current() { return 0; }\n}"), QStringLiteral("php"));
+
+    auto features = TopDUContext::AllDeclarationsAndContexts;
+
+    for (int i = 0; i < 2; ++i) {
+        if (i == 1) {
+            file.setFileContents(QStringLiteral("<?php\n/*\n*/\n\n/*\n*/\nforeach (new A() as $a) {}\nclass A implements Iterator {\npublic function current() { return 0; }\n}"));
+            features = static_cast<TopDUContext::Features>(features | TopDUContext::ForceUpdate);
+        }
+
+        file.parse(features);
+        QVERIFY(file.waitForParsed());
+
+        DUChainReadLocker lock;
+        auto top = file.topContext();
+        QVERIFY(top);
+        QVERIFY(top->localDeclarations().size() == 2);
+        QCOMPARE(top->localDeclarations().at(0)->qualifiedIdentifier(), QualifiedIdentifier("a"));
+
+        IntegralType::Ptr type = top->localDeclarations().at(0)->type<IntegralType>();
+        QVERIFY(type);
+        //Should actually parse as an TypeInt, but this does not work.
+        QVERIFY(type->dataType() == IntegralType::TypeMixed);
+    }
+}
