@@ -47,7 +47,7 @@ LexerTest::LexerTest()
 
 void LexerTest::testOpenTagWithNewline()
 {
-    TokenStream* ts = tokenize("<?php\nfoo;");
+    TokenStream* ts = tokenize(QStringLiteral("<?php\nfoo;"));
     QVERIFY(ts->size() == 3);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -59,7 +59,7 @@ void LexerTest::testOpenTagWithNewline()
 
 void LexerTest::testOpenTagWithSpace()
 {
-    TokenStream* ts = tokenize("<?php foo;");
+    TokenStream* ts = tokenize(QStringLiteral("<?php foo;"));
     QVERIFY(ts->size() == 3);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -70,7 +70,7 @@ void LexerTest::testOpenTagWithSpace()
 
 void LexerTest::testCommentOneLine()
 {
-    TokenStream* ts = tokenize("<?php\n//comment\nfoo;");
+    TokenStream* ts = tokenize(QStringLiteral("<?php\n//comment\nfoo;"));
     QVERIFY(ts->size() == 4);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -82,7 +82,7 @@ void LexerTest::testCommentOneLine()
 
 void LexerTest::testCommentOneLine2()
 {
-    TokenStream* ts = tokenize("<?php\n#comment\nfoo;");
+    TokenStream* ts = tokenize(QStringLiteral("<?php\n#comment\nfoo;"));
     QVERIFY(ts->size() == 4);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -94,7 +94,7 @@ void LexerTest::testCommentOneLine2()
 
 void LexerTest::testCommentMultiLine()
 {
-    TokenStream* ts = tokenize("<?php\n/*com\nment*/\nfoo;", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php\n/*com\nment*/\nfoo;"), true);
     QVERIFY(ts->size() == 5);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -107,7 +107,7 @@ void LexerTest::testCommentMultiLine()
 
 void LexerTest::testCommentMultiLine2()
 {
-    TokenStream* ts = tokenize("<?php\n/*\nment*/\nfoo;", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php\n/*\nment*/\nfoo;"), true);
     QVERIFY(ts->size() == 5);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -120,7 +120,7 @@ void LexerTest::testCommentMultiLine2()
 
 void LexerTest::testEndTag()
 {
-    TokenStream* ts = tokenize("<?\n':\n'?>\n>", true, Lexer::DefaultState);
+    TokenStream* ts = tokenize(QStringLiteral("<?\n':\n'?>\n>"), true, Lexer::DefaultState);
     //don't crash and we are fine
     delete ts;
 }
@@ -129,7 +129,7 @@ void LexerTest::testNewlineInString()
 {
     //0            1
     //012345 6 7 890123456789
-    TokenStream* ts = tokenize("<?php \"\n\";", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php \"\n\";"), true);
     QVERIFY(ts->size() == 3);
 
     COMPARE_TOKEN(ts, 1, Parser::Token_CONSTANT_ENCAPSED_STRING, 0, 6, 1, 0);
@@ -141,7 +141,7 @@ void LexerTest::testNewlineInString2()
 {
     //0
     //0123 4567
-    TokenStream* ts = tokenize("<?php '\n';", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php '\n';"), true);
     QCOMPARE((int)ts->size(), 3);
 
     COMPARE_TOKEN(ts, 1, Parser::Token_CONSTANT_ENCAPSED_STRING, 0, 6, 1, 0);
@@ -151,7 +151,7 @@ void LexerTest::testNewlineInString2()
 
 void LexerTest::testNewlineInStringWithVar()
 {
-    TokenStream* ts = tokenize("<?php \"$a\n\";", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php \"$a\n\";"), true);
     QCOMPARE((int)ts->size(), 6);
 
     COMPARE_TOKEN(ts, 1, Parser::Token_DOUBLE_QUOTE, 0, 6, 0, 6);
@@ -166,7 +166,7 @@ void LexerTest::testNewlineInStringWithVar2()
 {
     //0            1
     //012345 6 789 0123456789
-    TokenStream* ts = tokenize("<?php \"\n$a\n\";", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php \"\n$a\n\";"), true);
     QCOMPARE((int)ts->size(), 7);
 
     COMPARE_TOKEN(ts, 1, Parser::Token_DOUBLE_QUOTE, 0, 6, 0, 6);
@@ -182,7 +182,7 @@ void LexerTest::testNewlineInStringWithVar3()
 {
     //0            1
     //012345 6 789 0123456789
-    TokenStream* ts = tokenize("<?php \"{$$a}\";", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php \"{$$a}\";"), true);
     QCOMPARE((int)ts->size(), 7);
 
     COMPARE_TOKEN(ts, 1, Parser::Token_DOUBLE_QUOTE, 0, 6, 0, 6);
@@ -199,7 +199,7 @@ void LexerTest::testMultiplePhpSections()
 
     //0            1
     //012345 6 789 0123456789
-    TokenStream* ts = tokenize("<?php $a;?>\n<html>\n<?php $a;?>", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php $a;?>\n<html>\n<?php $a;?>"), true);
     QCOMPARE((int)ts->size(), 9);
 
     qint64 index = 0;
@@ -228,7 +228,7 @@ void LexerTest::testMultiplePhpSections()
 
 void LexerTest::testHereDoc()
 {
-    TokenStream* ts = tokenize("<?php\necho <<<EOD1\nstart $text\nend\nEOD1;\n$extern;", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php\necho <<<EOD1\nstart $text\nend\nEOD1;\n$extern;"), true);
     QCOMPARE((int)ts->size(), 12);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -246,7 +246,7 @@ void LexerTest::testHereDoc()
 
 void LexerTest::testHereDocQuoted()
 {
-    TokenStream* ts = tokenize("<?php\necho <<<\"EOD1\"\nstart $text\nend\nEOD1;\n$extern;", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php\necho <<<\"EOD1\"\nstart $text\nend\nEOD1;\n$extern;"), true);
     QCOMPARE((int)ts->size(), 12);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -264,7 +264,7 @@ void LexerTest::testHereDocQuoted()
 
 void LexerTest::testNowdoc()
 {
-    TokenStream* ts = tokenize("<?php\necho <<<'EOD1'\nstart $text\nend\nEOD1;\n$extern;", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php\necho <<<'EOD1'\nstart $text\nend\nEOD1;\n$extern;"), true);
     QCOMPARE((int)ts->size(), 10);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -296,7 +296,7 @@ void LexerTest::testCommonStringTokens()
 
 void LexerTest::testNonTerminatedStringWithVar()
 {
-    TokenStream* ts = tokenize("<?php \"$a", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php \"$a"), true);
 
     QCOMPARE((int)ts->size(), 3);
 
@@ -309,10 +309,10 @@ void LexerTest::testNonTerminatedStringWithVar()
 void LexerTest::testPhpBlockWithComment()
 {
     TokenStream* ts = tokenize(
-        "<?php\n"
+        QStringLiteral("<?php\n"
         "//asdf\n"
         "?>\n"
-        "<?php\n"
+        "<?php\n")
     , true);
 
     QCOMPARE((int)ts->size(), 5);
@@ -328,11 +328,11 @@ void LexerTest::testPhpBlockWithComment()
 void LexerTest::testNamespaces()
 {
     TokenStream* ts = tokenize(
-        "<?php\n"
+        QStringLiteral("<?php\n"
         "namespace Foo;\n"
         "namespace Foo\\Bar;\n"
         "namespace Foo\\Bar\\Asd {\n"
-        "}\n"
+        "}\n")
     , true);
     QCOMPARE((int)ts->size(), 25);
 
@@ -368,7 +368,7 @@ void LexerTest::testCloseTagInComment()
 {
     {
     TokenStream* ts = tokenize(
-        "<?php // asdf ?>"
+        QStringLiteral("<?php // asdf ?>")
     , true);
     QCOMPARE((int)ts->size(), 3);
 
@@ -380,7 +380,7 @@ void LexerTest::testCloseTagInComment()
     }
     {
     TokenStream* ts = tokenize(
-        "<?php #  asdf ?>"
+        QStringLiteral("<?php #  asdf ?>")
     , true);
     QCOMPARE((int)ts->size(), 3);
 
@@ -394,7 +394,7 @@ void LexerTest::testCloseTagInComment()
 
 void LexerTest::testBinaryNumber()
 {
-    TokenStream* ts = tokenize("<?php\n0b01;\n0B01;", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php\n0b01;\n0B01;"), true);
     QCOMPARE((int)ts->size(), 6);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -408,7 +408,7 @@ void LexerTest::testBinaryNumber()
 
 void LexerTest::testHexadecimalNumber()
 {
-    TokenStream* ts = tokenize("<?php\n0x01;\n0X01;\n0xABC12;\n0Xab10A;", true);
+    TokenStream* ts = tokenize(QStringLiteral("<?php\n0x01;\n0X01;\n0xABC12;\n0Xab10A;"), true);
     QCOMPARE((int)ts->size(), 12);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -428,7 +428,7 @@ void LexerTest::testHexadecimalNumber()
 
 void LexerTest::testTypeHintsOnFunction()
 {
-    TokenStream *ts = tokenize("<?php\nfunction a($a, array $b = [], callable $c) {}", true);
+    QScopedPointer<TokenStream> ts(tokenize(QStringLiteral("<?php\nfunction a($a, array $b = [], callable $c) {}"), true));
     QCOMPARE((int)ts->size(), 25);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -458,9 +458,28 @@ void LexerTest::testTypeHintsOnFunction()
     COMPARE_TOKEN(ts, 24, Parser::Token_RBRACE, 1, 44, 1, 44);
 }
 
+void LexerTest::testReturnTypeHints()
+{
+    QScopedPointer<TokenStream> ts(tokenize(QStringLiteral("<?php\nfunction a(): string {}"), true));
+    QCOMPARE((int)ts->size(), 12);
+
+    COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
+    COMPARE_TOKEN(ts, 1, Parser::Token_FUNCTION, 1, 0, 1, 7);
+    COMPARE_TOKEN(ts, 2, Parser::Token_WHITESPACE, 1, 8, 1, 8);
+    COMPARE_TOKEN(ts, 3, Parser::Token_STRING, 1, 9, 1, 9);
+    COMPARE_TOKEN(ts, 4, Parser::Token_LPAREN, 1, 10, 1, 10);
+    COMPARE_TOKEN(ts, 5, Parser::Token_RPAREN, 1, 11, 1, 11);
+    COMPARE_TOKEN(ts, 6, Parser::Token_COLON, 1, 12, 1, 12);
+    COMPARE_TOKEN(ts, 7, Parser::Token_WHITESPACE, 1, 13, 1, 13);
+    COMPARE_TOKEN(ts, 8, Parser::Token_STRING_TYPE, 1, 14, 1, 19);
+    COMPARE_TOKEN(ts, 9, Parser::Token_WHITESPACE, 1, 20, 1, 20);
+    COMPARE_TOKEN(ts, 10, Parser::Token_LBRACE, 1, 21, 1, 21);
+    COMPARE_TOKEN(ts, 11, Parser::Token_RBRACE, 1, 22, 1, 22);
+}
+
 void LexerTest::testExponentiation()
 {
-    TokenStream *ts = tokenize("<?php\n$a = 2 ** 3; $a **= 2;", true);
+    QScopedPointer<TokenStream> ts(tokenize(QStringLiteral("<?php\n$a = 2 ** 3; $a **= 2;"), true));
     QCOMPARE((int)ts->size(), 18);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -486,7 +505,7 @@ void LexerTest::testExponentiation()
 
 void LexerTest::testExceptionFinally()
 {
-    TokenStream *ts = tokenize("<?php\ntry { $a = 1; } finally { }", true);
+    QScopedPointer<TokenStream> ts(tokenize(QStringLiteral("<?php\ntry { $a = 1; } finally { }"), true));
     QCOMPARE((int)ts->size(), 19);
 
     COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
@@ -510,9 +529,63 @@ void LexerTest::testExceptionFinally()
     COMPARE_TOKEN(ts, 18, Parser::Token_RBRACE, 1, 26, 1, 26);
 }
 
+void LexerTest::testEllipsis()
+{
+    QScopedPointer<TokenStream> ts(tokenize(QStringLiteral("<?php\nfunction foo(...$args) {}"), true));
+    QCOMPARE((int)ts->size(), 11);
+
+    COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
+    COMPARE_TOKEN(ts, 1, Parser::Token_FUNCTION, 1, 0, 1, 7);
+    COMPARE_TOKEN(ts, 2, Parser::Token_WHITESPACE, 1, 8, 1, 8);
+    COMPARE_TOKEN(ts, 3, Parser::Token_STRING, 1, 9, 1, 11);
+    COMPARE_TOKEN(ts, 4, Parser::Token_LPAREN, 1, 12, 1, 12);
+    COMPARE_TOKEN(ts, 5, Parser::Token_ELLIPSIS, 1, 13, 1, 15);
+    COMPARE_TOKEN(ts, 6, Parser::Token_VARIABLE, 1, 16, 1, 20);
+    COMPARE_TOKEN(ts, 7, Parser::Token_RPAREN, 1, 21, 1, 21);
+    COMPARE_TOKEN(ts, 8, Parser::Token_WHITESPACE, 1, 22, 1, 22);
+    COMPARE_TOKEN(ts, 9, Parser::Token_LBRACE, 1, 23, 1, 23);
+    COMPARE_TOKEN(ts, 10, Parser::Token_RBRACE, 1, 24, 1, 24);
+}
+
+void LexerTest::testSpaceship()
+{
+    QScopedPointer<TokenStream> ts(tokenize(QStringLiteral("<?php\n$a = 'ab' <=> 'b';"), true));
+    QCOMPARE((int)ts->size(), 11);
+
+    COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
+    COMPARE_TOKEN(ts, 1, Parser::Token_VARIABLE, 1, 0, 1, 1);
+    COMPARE_TOKEN(ts, 2, Parser::Token_WHITESPACE, 1, 2, 1, 2);
+    COMPARE_TOKEN(ts, 3, Parser::Token_ASSIGN, 1, 3, 1, 3);
+    COMPARE_TOKEN(ts, 4, Parser::Token_WHITESPACE, 1, 4, 1, 4);
+    COMPARE_TOKEN(ts, 5, Parser::Token_CONSTANT_ENCAPSED_STRING, 1, 5, 1, 8);
+    COMPARE_TOKEN(ts, 6, Parser::Token_WHITESPACE, 1, 9, 1, 9);
+    COMPARE_TOKEN(ts, 7, Parser::Token_SPACESHIP, 1, 10, 1, 12);
+    COMPARE_TOKEN(ts, 8, Parser::Token_WHITESPACE, 1, 13, 1, 13);
+    COMPARE_TOKEN(ts, 9, Parser::Token_CONSTANT_ENCAPSED_STRING, 1, 14, 1, 16);
+    COMPARE_TOKEN(ts, 10, Parser::Token_SEMICOLON, 1, 17, 1, 17);
+}
+
+void LexerTest::testNullCoalesce()
+{
+    QScopedPointer<TokenStream> ts(tokenize(QStringLiteral("<?php\n$a = null ?? true;"), true));
+    QCOMPARE((int)ts->size(), 11);
+
+    COMPARE_TOKEN(ts, 0, Parser::Token_OPEN_TAG, 0, 0, 0, 5);
+    COMPARE_TOKEN(ts, 1, Parser::Token_VARIABLE, 1, 0, 1, 1);
+    COMPARE_TOKEN(ts, 2, Parser::Token_WHITESPACE, 1, 2, 1, 2);
+    COMPARE_TOKEN(ts, 3, Parser::Token_ASSIGN, 1, 3, 1, 3);
+    COMPARE_TOKEN(ts, 4, Parser::Token_WHITESPACE, 1, 4, 1, 4);
+    COMPARE_TOKEN(ts, 5, Parser::Token_STRING, 1, 5, 1, 8);
+    COMPARE_TOKEN(ts, 6, Parser::Token_WHITESPACE, 1, 9, 1, 9);
+    COMPARE_TOKEN(ts, 7, Parser::Token_NULL_COALESCE, 1, 10, 1, 11);
+    COMPARE_TOKEN(ts, 8, Parser::Token_WHITESPACE, 1, 12, 1, 12);
+    COMPARE_TOKEN(ts, 9, Parser::Token_STRING, 1, 13, 1, 16);
+    COMPARE_TOKEN(ts, 10, Parser::Token_SEMICOLON, 1, 17, 1, 17);
+}
+
 TokenStream* LexerTest::tokenize(const QString& unit, bool debug, int initialState)
 {
-    TokenStream* tokenStream = new TokenStream;
+    auto* tokenStream = new TokenStream;
     Lexer lexer(tokenStream, unit, initialState);
     int token;
     int i = 0;
@@ -533,8 +606,8 @@ TokenStream* LexerTest::tokenize(const QString& unit, bool debug, int initialSta
             qint64 endColumn;
             tokenStream->endPosition(i, &endLine, &endColumn);
             qDebug() << tokenText(t.kind)
-            << unit.mid(t.begin, t.end - t.begin + 1).replace('\n', "\\n")
-            << QString("[%0-%1] - [%2-%3]").arg(beginLine).arg(beginColumn).arg(endLine).arg(endColumn);
+            << unit.mid(t.begin, t.end - t.begin + 1).replace('\n', QLatin1String("\\n"))
+            << QStringLiteral("[%0-%1] - [%2-%3]").arg(beginLine).arg(beginColumn).arg(endLine).arg(endColumn);
             ++i;
         }
     }
