@@ -30,6 +30,7 @@
 #include <interfaces/ilanguagecontroller.h>
 #include <language/backgroundparser/backgroundparser.h>
 #include <language/backgroundparser/urlparselock.h>
+#include <language/editor/documentrange.h>
 
 #include "editorintegrator.h"
 #include "parsesession.h"
@@ -54,7 +55,7 @@ namespace Php
 
 ParseJob::ParseJob(const IndexedString& url, ILanguageSupport* languageSupport)
 : KDevelop::ParseJob(url, languageSupport)
-, m_parentJob(0)
+, m_parentJob(nullptr)
 {
 }
 
@@ -101,7 +102,7 @@ void ParseJob::run(ThreadWeaver::JobPointer /*self*/, ThreadWeaver::Thread * /*t
     session.setCurrentDocument(document());
 
     // 2) parse
-    StartAst* ast = 0;
+    StartAst* ast = nullptr;
     bool matched = session.parse(&ast);
 
     if (abortRequested() || ICore::self()->shuttingDown()) {
@@ -152,7 +153,7 @@ void ParseJob::run(ThreadWeaver::JobPointer /*self*/, ThreadWeaver::Thread * /*t
         }
 
         if (hadUnresolvedIdentifiers) {
-            if (!(minimumFeatures() & Resheduled) && KDevelop::ICore::self()->languageController()->backgroundParser()->queuedCount()) {
+            if (!(minimumFeatures() & Resheduled)) {
                 // Need to create new parse job with lower priority
                 qCDebug(PHP) << "Reschedule file " << document().str() << "for parsing";
                 KDevelop::TopDUContext::Features feat = static_cast<KDevelop::TopDUContext::Features>(
